use constant EOL => "\015\012";

my $dbhost          = $main::pluginconfig{'scripter'}{'database_host'}{''};
my $dbname          = $main::pluginconfig{'scripter'}{'database_name'}{''};
my $dbuser          = $main::pluginconfig{'scripter'}{'database_user'}{''};
my $dbpass          = $main::pluginconfig{'scripter'}{'database_password'}{''};
my $extensions_file = $main::pluginconfig{'scripter'}{'asterisk_extensions_conf_file'}{''};
my $scripter_file   = $main::pluginconfig{'scripter'}{'scripter_conf_file'}{''};

my $dbhscripter;

$AMI_Event_Handler{'scripter'}{'STATUSCOMPLETE'} = sub {

    my @allreturn;
    my $uid;
    my $gid;

    if( ! -f $scripter_file ) {
        (undef, undef, undef, undef, $uid, $gid, undef, undef, undef, undef, undef, undef) = stat ($extensions_file);
        open(my $fh,'>',$scripter_file);
        close($fh);
        my @files = ( $scripter_file );
        chown $uid, $gid, @files;
    }

    open my $fh, '<', $extensions_file;
    my $existe=0;
    while (my $line = <$fh>) {
        if ($line =~ /#include "$scripter_file"/) {
            $existe=1;
        }
    }

    if ($existe==0) {
        open(my $fh, '>>', $extensions_file);
        print $fh "#include \"$scripter_file\"\n";
        close $fh;
    }
    
    scripter_generate_dialplan();
    return @allreturn;
};

$AMI_Event_Handler{'scripter'}{'USEREVENT'} = sub {
    my $event = shift;
    my @allreturn;
    my $userevent = ${$event}{UserEvent};
    if ($userevent eq "Reload") {
        scripter_generate_dialplan();
    } elsif ($userevent eq "FOP2SCRIPTER") {
         my $context = ${$event}{Context};
         my $cmd     = ${$event}{Cmd};
         if($cmd eq "DelCat") {
             if($context =~ m/phonesurvey/) {
                 scripter_del_cat($context);
             }
         }
    }

    return @allreturn;
};

sub scripter_generate_dialplan {
    main::log_debug( "** SCRIPTER: regeneration dialplan\n", 128) if main::DEBUG;
    scripter_dialplan_surveys();
}

sub scripter_dialplan_answers {

    my @data = ();
    my $context;

    for(my $a=1;$a<10;$a++) {
        $context = "encuesta-answer-$a";
        push @data, "$context^delcat^none^none";
    }
    scripter_dialplan_append(\@data,'false');

    @data = ();
    for(my $a=1;$a<10;$a++) {
        $context = "encuesta-answer-$a";
        push @data, "$context^newcat^none^none";

        for(my $b=1;$b<=$a;$b++) {
            push @data, "$context^append^exten^> $b,1,Noop(Eligio $b)";
            push @data, "$context^append^exten^> $b,n,Set(RESP=\${CUT(ANSWER_VALUE,&,$b)})";
            push @data, "$context^append^exten^> $b,n,ExecIf(\$[\"\${RESP}\" = \"\"]?Set(RESP=$b))";
            push @data, "$context^append^exten^> $b,n,Set(PUSH(FINAL,|)=\${ANSFLDNAME}=\${RESP})";
            push @data, "$context^append^exten^> $b,n,Goto(\${NEXTSTEP})";
        }
    }

    scripter_dialplan_append(\@data,'false');
}

sub scripter_dialplan_surveys {

    scripter_dialplan_answers();
    scripter_dialplan_macro(); 

    my $dbhscripter = DBI->connect("DBI:mysql:$dbname:$dbhost", $dbuser, $dbpass, { mysql_enable_utf8 => 1 });

    if(!defined($dbhscripter)) { print "error dbh\n"; return; }

    $sth = $dbhscripter->prepare('SET NAMES utf8');
    $sth->execute();

    $query = "SELECT B.script_id,A.node_id,A.node_index,question,randomize_answer_order,next_node_id,type,";
    $query.= "group_concat(concat_ws('^',answer_id,answer_value,IFNULL(answer_link,'0')) ORDER BY C.sort,answer_id SEPARATOR '!') AS answers,";
    $query.= "group_concat(concat_ws('^',playback_id,playback_command,IFNULL(playback_data,'')) ORDER BY D.sort,playback_id SEPARATOR '!'), A.open_link, A.page_title ";
    $query.= "FROM asterisk.scripternodes A LEFT JOIN asterisk.scripterscripts B ON B.script_id=A.script_id ";
    $query.= "LEFT JOIN asterisk.scripteranswers C ON C.node_id=A.node_id LEFT JOIN scripterplaybacks D ON D.node_id=A.node_id ";
    $query.= "WHERE B.template='phone' GROUP BY B.script_id,A.node_index ORDER BY B.script_id,node_index";

    my $last_id=-1;
    my $a;

    my @answers;
    my @playbacks;
    my $context;

    # store mysql results in memory and disconnect quicly, ami operations are slow, we do not want to block mysql
    $sth  = $dbhscripter->prepare($query);
    $sth->execute();
    my @results = ();
    while(my @row = $sth->fetchrow_array) {
        push @results, \@row;
    }
    $dbhscripter->disconnect;

    my $howmany = @results;
    my @data = ();

    $count=0;
    my $id;
    foreach my $row (@results) {
        $count++;

        $id              = @$row[0];
        my $node_id      = @$row[1];
        my $node_index   = @$row[2];
        my $text         = @$row[3];
        my $random       = @$row[4];
        my $next_node_id = @$row[5];
        my $type         = @$row[6];
        my $answers      = @$row[7];
        my $playbacks    = @$row[8];
        my $open_link    = @$row[9];
        my $title        = @$row[10];

        $title =~ s/ /_/g;
        $title =~ s/[^a-zA-Z0-9_]+//g;

        if($id ne $last_id) {

            if($last_id != -1) {
                # close previeous survey macro
                $context = "phonesurvey-$last_id";
                push @data, "$context^append^exten^> s,n,Hangup";
                push @data, "$context^append^exten^> h,1,Noop(Main Hangup)";
                push @data, "$context^append^exten^> h,n,GotoIf(\$[\"\${FINAL}\" = \"\"]?end)";
                push @data, "$context^append^exten^> h,n,Set(REALFINAL=\${STRREPLACE(FINAL,~,)})";
                push @data, "$context^append^exten^> h,n,Set(SAVE=\${BASE64_ENCODE(\${REALFINAL})})";
                push @data, "$context^append^exten^> h,n,DumpChan";
                push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=UNIQUEID=\${UNIQUEID})";
                push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=QUEUENUM=\${QUEUENUM})";
                push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=CONNECTEDLINE=\${CONNECTEDLINE(all)})";
                push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=BRIDGEPEER=\${BRIDGEPEER})";
                push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=CALLERID=\${CALLERID(all)})";
                push @data, "$context^append^exten^> h,n,Set(SAVEMETADATA=\${BASE64_ENCODE(\${METADATA})})";
                push @data, "$context^append^exten^> h,n,System(/var/www/html/fop2/admin/plugins/scripter/savedata.php $last_id \${SAVE} \${SAVEMETADATA})";
                push @data, "$context^append^exten^> h,n(end),Noop(End)";
            }

            $context   = "phonesurvey-$id";
            scripter_del_cat($context);

            push @data, "$context^newcat^none^none";

            push @data, "$context^append^exten^> s,1,Answer()";
            push @data, "$context^append^exten^> s,n,gosub(app-split-dialervar,s,1)";

            $last_id = $id;
            $a=1;

        }

        if($random eq 'on') {
            $random=1;
        } else {
            $random=0;
        }

        my $mainaudio = 'scripter/'.sprintf("%08d",$node_id)."_00000000";
        if ( ! -f '/var/lib/asterisk/sounds/'.$mainaudio.'.wav' ) {
            $mainaudio = 'silence/1';
        }

        if($type eq 'question') {

            my @partes = split(/!/,$answers);

            my @audio;
            my @answer;
            my %link;
            my $i = 0;
            my $j = 0;

            for my $ans (@partes) {
                my @pertes = split(/\^/,$ans);
                my $answer_id    = @pertes[0];
                my $answer_value = @pertes[1];
                my $answer_link  = @pertes[2];

                my $audiofile = sprintf("%08d",$node_id)."_".sprintf("%08d",$answer_id);
                if ( -f '/var/lib/asterisk/sounds/scripter/'.$audiofile.'.wav' ) {
                    push @audio, 'scripter/'.$audiofile;
                } else {
                    push @audio, 'silence/1';
                }

                if($answer_value eq '') {
                    $j = $i;
                    $j++;
                    $answer_value=$j;
                }

                $answer_value =~ s/,/:/g;
                push @answer,   $answer_value;
                if($answer_link ne '') { $link{$answer_value} = $answer_link; }
                $i++;
                
            }

            if(scalar(@audio)>0) {

                my $allaudios = join("&",@audio);
                my $allanswers = join("&",@answer);
                my $answer_context = 'encuesta-answer-'.$i;

                my $actual  = "return_$a";
                my $curnode = "node_$a";
                my $retnode = $a+1;
                my $proximo = "return_$retnode";
                push @data, "$context^append^exten^> s,n($actual),Noop(Current Step: $actual)";
                push @data, "$context^append^exten^> s,n($curnode),Set(NEXTSTEP=phonesurvey-$id,s,$proximo)";

                push @data, "$context^append^exten^> s,n,Set(CURRENTDATA=\${FINAL})";
                push @data, "$context^append^exten^> s,n,macro(phonesurvey,$mainaudio,$allaudios,$allanswers,$answer_context,$random,$title)";
                push @data, "$context^append^exten^> s,n,ExecIf(\$[(\"\${CURRENTDATA}\" = \"\${FINAL}\")||(\"x\${CURRENTDATA}\" = \"x\")]?Set(PUSH(FINAL,|)=~))";

                push @data, "$context^append^exten^> s,n($proximo),Noop(Current Step: $proximo)";

                my $counter=0;
                while( my( $value, $gotonode ) = each %link ){
                    if($gotonode>0) {
                        if($counter==0) {
                            push @data, "$context^append^exten^> s,n,Set(REALFINAL=\${STRREPLACE(FINAL,~,)})";
                            push @data, "$context^append^exten^> s,n,Set(SAVE=\${BASE64_ENCODE(\${REALFINAL})})";
                            push @data, "$context^append^exten^> s,n,Set(PUSH(METADATA,|)=UNIQUEID=\${UNIQUEID})";
                            push @data, "$context^append^exten^> s,n,Set(PUSH(METADATA,|)=QUEUENUM=\${QUEUENUM})";
                            push @data, "$context^append^exten^> s,n,Set(PUSH(METADATA,|)=CONNECTEDLINE=\${CONNECTEDLINE(all)})";
                            push @data, "$context^append^exten^> s,n,Set(PUSH(METADATA,|)=BRIDGEPEER=\${BRIDGEPEER})";
                            push @data, "$context^append^exten^> s,n,Set(PUSH(METADATA,|)=CALLERID=\${CALLERID(all)})";
                            push @data, "$context^append^exten^> s,n,Set(SAVEMETADATA=\${BASE64_ENCODE(\${METADATA})})";
                            push @data, "$context^append^exten^> s,n,System(/var/www/html/fop2/admin/plugins/scripter/savedata.php $last_id \${SAVE} \${SAVEMETADATA})";
                        }
                        $counter++;

                        push @data, "$context^append^exten^> s,n,GotoIf(\$[\"\${RESP}\" = \"$value\"]?node_$gotonode)";
                    }
                }
            }

        } elsif ($type eq 'solution') {

            my $curnode="node_$a";
            my $checkfile = '/var/lib/asterisk/sounds/'.$mainaudio.'.wav';

            if(-f $checkfile) {
                push @data, "$context^append^exten^> s,n($curnode),Playback($mainaudio)";
            } else {
                push @data, "$context^append^exten^> s,n($curnode),Noop(No main audio for this node)";
            }

            my $datavalue='';
            my @partes = split(/!/,$playbacks);
            for my $pb (@partes) {
                my @pertes = split(/\^/,$pb);
                $play_id = @pertes[0];
                $command = @pertes[1];
                $datavalue    = @pertes[2];
                $datavalue    =~ s/^\s+//;
                $datavalue    =~ s/\s+$//;
                $play_id = $play_id + 10000000;

                if($command eq 'playback') {
                    $sndfile=sprintf('%08d',$node_id).'_'.sprintf('%08d',$play_id);
                    $chkfile='/var/lib/asterisk/sounds/scripter/'.$sndfile.'.wav';
                    if(-f $chkfile) {
                        $datavalue='scripter/'.$sndfile;
                    }
                } elsif($command=='saynumber') {

                    if(substr($datavalue,0,2) eq '${' && substr($datavalue,-1) eq '}') {
                        $novar = substr($datavalue,2,-1);
                        push @data, "$context^append^exten^> s,n,ExecIf(\$[\"$datavalue\" = \"\"]?Set($novar=0))";
                    }
                } elsif($command eq 'tts') {
                    push @data, "$context^append^exten^> s,n,Set(text0=$datavalue)";
                    $command='agi';
                    $datavalue='tts_ibm.sh';
                }
                push @data, "$context^append^exten^> s,n,$command($datavalue)";
            }
        } elsif ($type eq 'link') {
            my $curnode="node_$a";

            my $checkfile = '/var/lib/asterisk/sounds/'.$mainaudio.'.wav';

            if(-f $checkfile) {
                push @data, "$context^append^exten^> s,n($curnode),Playback($mainaudio)";
            } else {
                push @data, "$context^append^exten^> s,n($curnode),Noop(No main audio for this node)";
            }

            my ($extension, $gotocontext) = split(/\@/,$open_link);
            push @data,"$context^append^exten^> s,n($curnode),Goto($gotocontext,$extension,1)";
            push @data,"$context^append^exten^> s,n,Hangup";
        }

        $a++;

        if($next_node_id>0) {
            if($next_node_id==999) {
                push @data, "$context^append^exten^> s,n,Hangup";
            } else {
                push @data, "$context^append^exten^> s,n,Goto(node_".$next_node_id.")";
            }
        }

    }

    scripter_dialplan_append(\@data,'false');

    # last survey generated, perform dialplan reload
    @data = ();
    push @data, "$context^append^exten^> s,n,Hangup";
    push @data, "$context^append^exten^> h,1,Noop(Main Hangup)";
    push @data, "$context^append^exten^> h,n,GotoIf(\$[\"\${FINAL}\" = \"\"]?end)";
    push @data, "$context^append^exten^> h,n,Set(REALFINAL=\${STRREPLACE(FINAL,~,)})";
    push @data, "$context^append^exten^> h,n,Set(SAVE=\${BASE64_ENCODE(\${REALFINAL})})";
    push @data, "$context^append^exten^> h,n,DumpChan";
    push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=UNIQUEID=\${UNIQUEID})";
    push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=QUEUENUM=\${QUEUENUM})";
    push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=CONNECTEDLINE=\${CONNECTEDLINE(all)})";
    push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=BRIDGEPEER=\${BRIDGEPEER})";
    push @data, "$context^append^exten^> h,n,Set(PUSH(METADATA,|)=CALLERID=\${CALLERID(all)})";
    push @data, "$context^append^exten^> h,n,Set(SAVEMETADATA=\${BASE64_ENCODE(\${METADATA})})";
    push @data, "$context^append^exten^> h,n,System(/var/www/html/fop2/admin/plugins/scripter/savedata.php $id \${SAVE} \${SAVEMETADATA})";
    push @data, "$context^append^exten^> h,n(end),Noop(End)";

    scripter_dialplan_append(\@data, 'true');

}

sub scripter_new_cat {

    my $context = shift;

    foreach my $manager_host ( keys %main::managers ) {
        if ( $main::astman{$manager_host}->{LOGGEDIN} ) {

            scripter_send_command_to_manager( $main::astman{$manager_host}, {
                  Action      => 'updateconfig',
                  SrcFilename => "$scripter_file",
                  DstFilename => "$scripter_file",
                  'Action-000000' => 'newcat',
                  'Cat-000000' => "$context"
            }, 'scripterDIALPLAN!a!a' );
        }
    }
}

sub scripter_del_cat {

    my $context = shift;

    foreach my $manager_host ( keys %main::managers ) {
        if ( $main::astman{$manager_host}->{LOGGEDIN} ) {

            scripter_send_command_to_manager( $main::astman{$manager_host}, {
                  Action      => 'updateconfig',
                  SrcFilename => "$scripter_file",
                  DstFilename => "$scripter_file",
                  'Action-000000' => 'delcat',
                  'Cat-000000' => "$context"
            }, 'scripterDIALPLAN!a!a' );
        }
    }
}

sub scripter_dialplan_append {

    my $value   = shift;
    my $reload  = shift;

    foreach my $manager_host ( keys %main::managers ) {
        if ( $main::astman{$manager_host}->{LOGGEDIN} ) {

            my $cmd='';
            $cmd.= 'Action: updateconfig'.EOL;
            if($reload eq  'true') {
                $cmd.= 'Reload: true'.EOL;
            }
            $cmd.='SrcFilename: '.$scripter_file.EOL;
            $cmd.='DstFilename: '.$scripter_file.EOL;

            $cnt=0;
            foreach my $val (@$value) {

                my @partes = split(/\^/,$val,4);
                my $context = @partes[0];
                my $act     = @partes[1];
                my $var     = @partes[2];
                my $val     = @partes[3];

                $cntprint = sprintf('%06d',$cnt);
                $cmd.='Action-'.$cntprint.': '.$act.EOL;
                $cmd.='Cat-'.$cntprint.': '.$context.EOL;

                if($act ne 'delcat' && $act ne 'newcat') {
                    $cmd.='Var-'.$cntprint.': '.$var.EOL;
                    $cmd.='Value-'.$cntprint.': '.$val.EOL;
                }

                $cnt++;

                if($cnt % 15 == 0) {

                    main::push_write_to_manager( $main::astman{$manager_host}, $cmd );

                    $cmd = '';

                    $cmd.= 'Action: updateconfig'.EOL;
                    if($reload eq  'true') {
                        $cmd.= 'Reload: true'.EOL;
                    }
                    $cmd.='SrcFilename: '.$scripter_file.EOL;
                    $cmd.='DstFilename: '.$scripter_file.EOL;

                    $cnt=0;

                }
            }

            if($cnt>0) {
                main::push_write_to_manager( $main::astman{$manager_host}, $cmd );
            }
        }

    }
}

sub scripter_dialplan_macro {

    main::log_debug( "** SCRIPTER: generating dialplan", 1 ) if main::DEBUG;

my $macroPhoneSurvey = '
exten => s,1,Noop(Phone Survey)
exten => s,n,Set(TIMEOUT(digit)=5)
exten => s,n,Set(MAIN_AUDIO=${ARG1})
exten => s,n,Set(ANSWER_AUDIO=${ARG2})
exten => s,n,Set(ANSWER_VALUE=${ARG3})
exten => s,n,Set(VALID=${ARG4})
exten => s,n,Set(RANDOM=${ARG5})
exten => s,n,Set(_ANSFLDNAME=${ARG6})
exten => s,n,GosubIf($["${RANDOM}"!="0"]?random)
exten => s,n,Noop(Sigo)
exten => s,n,Set(ITER=1)
exten => s,n,Set(LOOPCNT=3)
exten => s,n,Playback(${MAIN_AUDIO})
exten => s,n(ask),BackGround(${ANSWER_AUDIO}&silence/5,m,,${VALID})
exten => s,n,ExecIf($["${BACKGROUNDSTATUS}"="FAILED"]?MacroExit())
exten => s,n,Set(ITER=$[${ITER}+1])
exten => s,n,GotoIf($[${ITER}<=${LOOPCNT}]?ask)
exten => s,n,MacroExit
exten => s,n(random),Noop(do random)
exten => s,n,Set(I=1)
exten => s,n(field),Set(CNT=${FIELDQTY(ANSWER_AUDIO,&)})
exten => s,n,Noop(${CNT})
exten => s,n,GotoIf($["${CNT}"="0"]?fin)
exten => s,n,Set(NEEDLE=${RAND(1,${CNT})})
exten => s,n,Set(ITEM_AUDIO=${CUT(ANSWER_AUDIO,&,${NEEDLE})})
exten => s,n,Set(ITEM_VALUE=${CUT(ANSWER_VALUE,&,${NEEDLE})})
exten => s,n,Set(NEW_AUDIO=${STRREPLACE(ANSWER_AUDIO,${ITEM_AUDIO}&,)})
exten => s,n,Set(NEW_AUDIO=${STRREPLACE(NEW_AUDIO,&${ITEM_AUDIO},)})
exten => s,n,Set(NEW_AUDIO=${STRREPLACE(NEW_AUDIO,${ITEM_AUDIO},)})
exten => s,n,Set(ANSWER_AUDIO=${NEW_AUDIO})
exten => s,n,Set(NEW_VALUE=${STRREPLACE(ANSWER_VALUE,${ITEM_VALUE}&,)})
exten => s,n,Set(NEW_VALUE=${STRREPLACE(NEW_VALUE,&${ITEM_VALUE},)})
exten => s,n,Set(NEW_VALUE=${STRREPLACE(NEW_VALUE,${ITEM_VALUE},)})
exten => s,n,Set(ANSWER_VALUE=${NEW_VALUE})
exten => s,n,Set(FINAL_AUDIO=${FINAL_AUDIO}&${ITEM_AUDIO})
exten => s,n,Set(FINAL_VALUE=${FINAL_VALUE}&${ITEM_VALUE})
exten => s,n,Set(AUDIOFILE=/var/lib/asterisk/sounds/scripter/digits/${I}.wav)
exten => s,n,GotoIf($[${STAT(f,${AUDIOFILE})} = 1]?existe)
exten => s,n,Noop(Audio ${AUDIOFILE} does not exist, play stock digit audio)
exten => s,n,Set(FINAL_AUDIO=${FINAL_AUDIO}&digits/${I})
exten => s,n,Goto(sigue)
exten => s,n(existe),Noop(Audio ${AUDIOFILE} exists, use custom digit audio)
exten => s,n,Set(FINAL_AUDIO=${FINAL_AUDIO}&scripter/digits/${I})
exten => s,n(sigue),Set(I=$[${I} + 1])
exten => s,n,Goto(field)
exten => s,n(fin),Set(ANSWER_AUDIO=${FINAL_AUDIO:1})
exten => s,n,Set(ANSWER_VALUE=${FINAL_VALUE:1})
exten => s,n,Return
';

my $app_split_dialervar = '
exten => s,1,Set(CNT=${FIELDQTY(DIALERVAR,\|)})
exten => s,n(next),Noop(${CNT})
exten => s,n,Set(ELEM=${CUT(DIALERVAR,|,${CNT})})
exten => s,n,Set(VAR=${CUT(ELEM,=,1)})
exten => s,n,Set(VAL=${CUT(ELEM,=,2)})
exten => s,n,GotoIf($[${CNT} <= 0]?fin)
exten => s,n,Set(CNT=$[${CNT} - 1])
exten => s,n,GotoIf($["${VAL}"=""]?next)
exten => s,n,Set(DIALER_${VAR}=${VAL})
exten => s,n,Goto(next)
exten => s,n(fin),Return
';  

    my $context = 'macro-phonesurvey';
    scripter_del_cat($context);
    scripter_new_cat($context);

    my $splitcontext = 'app-split-dialervar';
    scripter_del_cat($splitcontext);
    scripter_new_cat($splitcontext);

    foreach my $manager_host ( keys %main::managers ) {

        if ( $main::astman{$manager_host}->{LOGGEDIN} ) {

            push @data, "$context^EmptyCat^none^none";
    
            my @lineas = split(/\n/,$macroPhoneSurvey);
            foreach my $linea (@lineas) {
                my $value = substr($linea,7);
                $value =~ s/^\s+//;
                $value =~ s/\s+$//;
                if($value eq '') { next; }
                push @data, "$context^append^exten^$value";
            }

           push @data, "$splitcontext^EmptyCat^none^none";

            my @lineas = split(/\n/,$app_split_dialervar);
            foreach my $linea (@lineas) {
                my $value = substr($linea,7);
                $value =~ s/^\s+//;
                $value =~ s/\s+$//;
                if($value eq '') { next; }
                push @data, "$splitcontext^append^exten^$value";
            }

            scripter_dialplan_append(\@data,'false');
        }
    }
}

sub scripter_send_command_to_manager {

    my ( $asterisk, $comando, $extra ) = @_;

    if ( !defined($extra) ) { $extra = ''; }

    if ( !defined($asterisk) ) { return; }

    $asterisk->send_action(
        $comando,
        sub {
            my ( $asterisk, $response ) = @_;
            \&scripter_handle_ami_response( $asterisk, $response, $extra );
        },
        0
    );
}

sub scripter_handle_ami_response {

    return if $expired;

    my ( $asterisk, $response, $extra ) = @_;

    my @return;

    if ( defined($extra) && $extra ne "" ) {

        my ( undef, undef, $finish ) = split( /\!/, $extra );

        main::log_debug( "** SCRIPTER: handler ami response $extra", 128) if main::DEBUG;

        if($finish eq "yes") {
#print "disconnect\n";
        }

    }

}
